#!/bin/sh

if ! which realpath > /dev/null ; then
    echo "Please install the realpath program." 1>&2
    exit 1
fi

THISDIR=`dirname "$0"`
DIRNAME=`realpath "$THISDIR/.."`
PROGNAME=`basename "$0"`

if [ "$1" = '--help' ]; then
    cat 1>&2 <<EOF
Usage: $PROGNAME [ --import | --show | --unchanged ] [ <filename> ... ]

Check the specified patch files or all files ending with .diff in the current
directory against the existing patch files in the patch directory:

$DIRNAME

Without any options, show the names of all differing patch files plus any
unknown files. The following options change the behaviour as follows:

--import    Import the specified patch files
--show      Show the details of each changed patch file
--unchanged Show the names of unchanged patch files

The --unchanged option is useful for removing local patch files as follows:

$THISDIR/$PROGNAME --unchanged | xargs rm
EOF
    exit 1
fi

if [ "$1" = '--import' ] ; then
    IMPORT="$1"
    shift 1
elif [ "$1" = '--show' ] ; then
    SHOW="$1"
    shift 1
elif [ "$1" = '--unchanged' ] ; then
    UNCHANGED="$1"
    shift 1
fi

# Use specified filenames or all patch files in the current directory.

if [ ! "$1" ]; then
    FILENAMES=*.diff
else
    FILENAMES=$*
fi

# Compare each filename to the existing patch files.

for FILENAME in $FILENAMES ; do
    EXISTING="$DIRNAME/"`basename "$FILENAME"`

    # Handle previously non-existent patch files.

    if [ ! -e "$EXISTING" ] ; then

        # Import such files.

        if [ "$IMPORT" ] ; then
            echo "Importing new $FILENAME" 1>&2
            mv "$FILENAME" "$EXISTING"

        # Otherwise, show them.

        elif [ ! "$SHOW" ] && [ ! "$UNCHANGED" ] ; then
            echo "$FILENAME"
        fi

    # Show full details if requested.

    elif [ "$SHOW" ] ; then
        diff "$EXISTING" "$FILENAME"

    # Import the patch file if requested.

    elif [ "$IMPORT" ] ; then
        mv "$FILENAME" "$EXISTING"

    # Show differing patch files.

    elif [ ! "$UNCHANGED" ] && ! diff -q "$EXISTING" "$FILENAME" > /dev/null 2>&1 ; then
        echo "$FILENAME"

    # Show unchanged patch files.

    elif [ "$UNCHANGED" ] && diff -q "$EXISTING" "$FILENAME" > /dev/null 2>&1 ; then
        echo "$FILENAME"
    fi
done
