#!/bin/sh

THISDIR=`dirname "$0"`
DIRNAME="$THISDIR"/..
PROGNAME=`basename "$0"`

if [ "$1" = '--help' ]; then
    cat 1>&2 <<EOF
Usage: $PROGNAME [ - ]

Produce recomputed patch files in the current working directory. These can then
be compared to those already available.

If - is specified, only produce patch files whose names are provided via
standard input. This is useful when only certain files have been edited and
only the corresponding patch files will need to be updated. Use the findpatch.sh
script in combination with this one in this situation:

  $THISDIR/findpatch.sh conf/examples/mips-qi_lb60-spectrum.* \\
| sort -u \\
| $THISDIR/$PROGNAME -

A file called new.diff will be produced containing all patches that cannot be
placed in existing patch files because the files involved are not associated
with any of these existing patch files. Such patches may either be included in
existing patch files or they may form the basis of new patch files to be added
to those already available.

Two files, a.txt and b.txt, respectively provide the list of files tracked by
existing patch files and the list of files changed in the source code
repository.
EOF
    exit 1
fi

# Read filenames from standard input if indicated. Otherwise scan the patch
# directory.

if [ "$1" = '-' ]; then
    read FILENAMES
else
    FILENAMES="$DIRNAME"/*.diff
fi

# Make new versions of each patch.

echo -n > a.txt.init

for FILENAME in $FILENAMES ; do
    LEAFNAME=`basename "$FILENAME"`

    # Obtain patch files from the collection if not found.

    if [ ! -e "$FILENAME" ] ; then
        FILENAME="$DIRNAME/$LEAFNAME"

        if [ ! -e "$FILENAME" ] ; then
            echo "$FILENAME not recognised." 1>&2
            continue
        fi
    fi

    # Make a patch file using differences affecting the files it mentions.

    grep ^Index "$FILENAME" | cut -d ' ' -f 2 | xargs -I{} svn diff '{}' > "$LEAFNAME.new"
    mv "$LEAFNAME.new" "$LEAFNAME"

    # Record all affected files.

    grep ^Index "$FILENAME" | cut -d ' ' -f 2 >> a.txt.init
done

# Find all files in patches and all those changed in the repository.

sort -u a.txt.init > a.txt
svn diff | grep ^Index | cut -d ' ' -f 2 | sort -u > b.txt
rm a.txt.init

# Produce a patch for all new files.

diff a.txt b.txt | grep '^>' | cut -d ' ' -f 2 | xargs -I{} svn diff '{}' > new.diff
